package tp;

import java.io.IOException;
import java.util.Locale;

import org.graphstream.algorithm.APSP;
import org.graphstream.algorithm.APSP.APSPInfo;
import org.graphstream.algorithm.Dijkstra;
import org.graphstream.algorithm.Toolkit;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.Path;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.GraphParseException;

public class Main {

	private static final String RESOURCE_DIRECTORY = "./resources/";

	public static void main(String[] args) {
		Graph g = new SingleGraph("lh");
		
		try {
            g.read(RESOURCE_DIRECTORY + "lh.dgs");
        } catch (IOException e) {
            e.printStackTrace();
        } catch (GraphParseException e) {
            e.printStackTrace();
		}
        
		g.addAttribute("ui.stylesheet",
		"node {size: 3px; fill-color: #777; text-mode: hidden; z-index: 0;} " +
		"edge { shape: line; fill-color: #222; arrow-size: 3px, 2px;}");
		g.addAttribute("ui.quality");
		g.addAttribute("ui.antialias");

		g.display(false);

		//ALGORITHMS FEST
		
		System.out.println("Comparaison d'algorithmes de recherche de plus court chemin:\n");
		compareShortestPathAlgorithms(g);
		
		//DATA GATHERING
		
		int nodeCount = g.getNodeCount();
		int edgeCount = g.getEdgeCount();
		
		double avgDegree = Toolkit.averageDegree(g);
		double clusteringCoeff = Toolkit.averageClusteringCoefficient(g);
		int[] distribution = Toolkit.degreeDistribution(g);	
		
		//DATA PRINT
		
		System.out.format(
			Locale.FRENCH, "Mesures du graphe:\n\n\tNombre de noeuds: %d\n\tNombres d'arcs: %d\n\tDegré moyen: %.3f\n\tCoefficient de clustering: %.3f\n\n",
			nodeCount, edgeCount, avgDegree, clusteringCoeff
		);

		System.out.println("Distribution des degrés:");
		for(int i = 0; i < distribution.length; i++) {
			System.out.format(Locale.FRENCH, "\t%d: %d\n", i, distribution[i]);
		}
		System.out.println();
	}
	
	/**
	 * Gives the shortest path for two given nodes in a given graph
	 * @param graph The graph in which computes the algorithm
	 * @param source The source node for the shortest path
	 * @param dest The destination node for the shortest path
	 * @return The Path object representing the shortest path
	 */
	public static Path shortestPath(Graph graph, Node source, Node dest) {
		Dijkstra dAlgo = new Dijkstra(Dijkstra.Element.EDGE, "result", "length");
		
		// Compute the shortest paths in g from A to all nodes
		dAlgo.init(graph);
		dAlgo.setSource(source);
		dAlgo.compute();
		              
		return dAlgo.getPath(dest);
	}
	

	/**
	 * Computes the diameter of a given graph using the Dijkstra algorithm
	 * @param g The graph from which the diameter will be computed
	 * @return The Path object representing the shortest path
	 */
	public static Path computeDiameterDijkstra(Graph g) {
		Node source = null;
		Node dest = null;
		double longest = 0;
		
		for(Node n : g.getEachNode()) {
			Dijkstra dAlgo = new Dijkstra(Dijkstra.Element.EDGE, "result", "length");
			
			// Compute the shortest paths in g from A to all nodes
			dAlgo.init(g);
			dAlgo.setSource(n);
			dAlgo.compute();
			
			for(Node n2 : g.getEachNode()) {
				double wgt = dAlgo.getPathLength(n2);
				if(wgt > longest) {
					longest = wgt;
					source = n;
					dest = n2;
				}
			}
		}
		
		Path p = shortestPath(g, source, dest);
		
		System.out.println("\nDiametre: " + longest);

		return p;
	}
	

	/**
	 * Computes the diameter of a given graph using the Floyd-Warshall algorithm
	 * @param g The graph from which the diameter will be computed
	 * @return The Path object representing the shortest path
	 */
	public static Path computeDiameterFloydWarshall(Graph g) {
		Path path = null;
		double longest = 0;
		
		APSP apsp = new APSP();
        apsp.init(g); 
        apsp.setDirected(false); 
        apsp.setWeightAttributeName("length");
        apsp.compute();

        for(Node n : g.getEachNode()) {
        	APSPInfo info = n.getAttribute(APSPInfo.ATTRIBUTE_NAME);
        	
			for(Node n2 : g.getEachNode()) {
				Path p = info.getShortestPathTo(n2.getId());
				double wgt = p.getPathWeight("length");

				if(wgt > longest) {
					longest = wgt;
					path = p;
				}
			}
		}
        
		System.out.println("\nDiametre: " + longest);
		
		return path;
	}

	/**
	 * Prints an execution time comparison for shortest paths algorithms, specifically to find the diameter of a given graph
	 * @param g The graph from whixh the diameter will be computed
	 */
	public static void compareShortestPathAlgorithms(Graph g) {
		long startTime = System.currentTimeMillis();
		//computeDiameterFloydWarshall(g); 
		long endTime = System.currentTimeMillis();

		//System.out.println("Temps avec Floyd-Warshall: " + (endTime - startTime) + " millisecondes");
		
		startTime = System.currentTimeMillis();
		Path p = computeDiameterDijkstra(g);
		endTime = System.currentTimeMillis();
		
		System.out.println("Temps avec Djikstra: " + (endTime - startTime) + " millisecondes");

		p.getNodeSet().forEach(n -> n.addAttribute("ui.style", "fill-color: darkgreen;"));
		p.getEdgeSet().forEach(n -> n.addAttribute("ui.style", "fill-color: green;"));
	}
}
